	-- TASKING HDL generator v1.3r1 Build 088
	-- options: -f Output\ChcOutputs\Embedded_TP3.vhd.opt --toplevel=Configurable_WB_ASP_1 --wishbone-address-width=4 --vhdl=Output\ChcOutputs\..\Embedded_TP3.vhd --verilog=Output\ChcOutputs\..\Embedded_TP3.v

	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;

	package Configurable_WB_ASP_1_chc_pkg is
		function bool2std_logic ( bool : boolean ) return std_logic;
		function vectorize_bit ( bit : std_logic ) return std_logic_vector;
		function devectorize_bit ( vec : std_logic_vector ) return std_logic;
		function extract ( vec : std_logic_vector; high, low : integer ) return std_logic_vector;
		function has_ux( v : std_logic_vector ) return boolean;
		function vec2str(vec: std_logic_vector) return string;
	end Configurable_WB_ASP_1_chc_pkg;

	package body Configurable_WB_ASP_1_chc_pkg is
		function bool2std_logic ( bool : boolean ) return std_logic is
		begin
			if (bool) then
				return '1';
			else
				return '0';
			end if;
		end bool2std_logic;

		function vectorize_bit ( bit : std_logic ) return std_logic_vector is
		  variable ret : std_logic_vector( 0 downto 0 );
		begin
			ret(0) := bit;
			return ret;
		end vectorize_bit;

		function devectorize_bit ( vec : std_logic_vector ) return std_logic is
		  alias v : std_logic_vector(vec'length-1 downto 0) is vec;
		  variable ret : std_logic;
		begin
			ret := v(0);
			return ret;
		end devectorize_bit;

		function extract ( vec : std_logic_vector; high, low : integer ) return std_logic_vector is
		  alias v : std_logic_vector(vec'length-1 downto 0) is vec;
		  variable ret         : std_logic_vector( high downto low );
		begin
		    ret := v( high downto low );
		    return ret;
		end extract;

		function has_ux( v : std_logic_vector ) return boolean is
		begin
		  for i in v'range loop
			if v(i) /= '0' and v(i) /= '1' then return true; end if;
		  end loop;
		  return false;
		end has_ux;

		function vec2str( vec: std_logic_vector ) return string is
		  alias v : std_logic_vector(vec'length-1 downto 0) is vec;
		  constant hexcodes : string( 1 to 16 ) := "0123456789ABCDEF" ;
		  variable nibbles : integer := ( v'high + 4 ) / 4;
		  variable temp : string( 1 to nibbles );
		  variable nibble : std_logic_vector(3 downto 0);
		  variable nibvec : std_logic_vector( nibbles * 4 - 1 downto 0 ) := ( others => '0' );
		begin
			nibvec( v'high downto 0 ) := v;
			for i in nibbles - 1 downto 0 loop 
				nibble := extract( nibvec, 4*i+3, 4*i );
				if has_ux( nibble ) then
					temp(nibbles-i) := 'X';
				else
					temp(nibbles-i) := hexcodes( 1 + to_integer( unsigned(nibble) mod 16 ));
				end if;
			end loop;
			return temp;
		end vec2str;
	end Configurable_WB_ASP_1_chc_pkg;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity Configurable_WB_ASP_1 is
port(
	io_ACK_O : out std_logic;
	io_ADR_I : in std_logic_vector( 3 downto 0 );
	io_CLK_I : in std_logic;
	io_CYC_I : in std_logic;
	io_DAT_I : in std_logic_vector( 31 downto 0 );
	io_DAT_O : out std_logic_vector( 31 downto 0 );
	io_RST_I : in std_logic;
	io_SEL_I : in std_logic_vector( 3 downto 0 );
	io_STB_I : in std_logic;
	io_WE_I : in std_logic;
	me_ACK_I : in std_logic;
	me_ADR_O : out std_logic_vector( 31 downto 0 );
	me_CLK_O : out std_logic;
	me_CYC_O : out std_logic;
	me_DAT_I : in std_logic_vector( 31 downto 0 );
	me_DAT_O : out std_logic_vector( 31 downto 0 );
	me_RST_O : out std_logic;
	me_SEL_O : out std_logic_vector( 3 downto 0 );
	me_STB_O : out std_logic;
	me_WE_O : out std_logic
);
end entity Configurable_WB_ASP_1;

architecture rtl of Configurable_WB_ASP_1 is
	component chc_Configurable_WB_ASP_1_crc16 is
	port(
		clock : in std_logic;
		count : in std_logic_vector( 31 downto 0 );
		done : out std_logic;
		me_ADR : out std_logic_vector( 29 downto 0 );
		me_DIN : out std_logic_vector( 31 downto 0 );
		me_DONE : in std_logic;
		me_DOUT : in std_logic_vector( 31 downto 0 );
		me_EN : out std_logic;
		me_SEL : out std_logic_vector( 3 downto 0 );
		me_WE : out std_logic;
		ptr : in std_logic_vector( 31 downto 0 );
		reset : in std_logic;
		retval : out std_logic_vector( 15 downto 0 );
		start : in std_logic
	);
	end component chc_Configurable_WB_ASP_1_crc16;

	signal asp_start : std_logic_vector( 31 downto 0 ) := X"00000000";
	signal busy_reg : std_logic_vector( 31 downto 0 ) := X"00000000";
	signal crc16_done : std_logic;
	signal crc16_retval : std_logic_vector( 15 downto 0 );
	signal crc16_start : std_logic;
	signal input_reg : std_logic_vector( 63 downto 0 ) := X"0000000000000000";
	signal input_start : std_logic_vector( 31 downto 0 ) := X"00000000";
	signal me_0_ADR : std_logic_vector( 29 downto 0 );
	signal me_0_DIN : std_logic_vector( 31 downto 0 );
	signal me_0_DONE : std_logic;
	signal me_0_DOUT : std_logic_vector( 31 downto 0 );
	signal me_0_EN : std_logic;
	signal me_0_SEL : std_logic_vector( 3 downto 0 );
	signal me_0_WE : std_logic;
	signal me_wbreg_adr : std_logic_vector( 29 downto 0 ) := "000000000000000000000000000000";
	signal me_wbreg_adr_lsb_reg : std_logic_vector( 1 downto 0 ) := "00";
	signal me_wbreg_adr_lsb_wire : std_logic_vector( 1 downto 0 ) := "00";
	signal me_wbreg_cyc : std_logic := '0';
	signal me_wbreg_in : std_logic_vector( 31 downto 0 ) := X"00000000";
	signal me_wbreg_sel : std_logic_vector( 3 downto 0 ) := "0000";
	signal me_wbreg_we : std_logic := '0';
	signal output_reg : std_logic_vector( 63 downto 0 ) := X"0000000000000000";
	alias crc16_count : std_logic_vector( 31 downto 0 ) is input_reg( 63 downto 32 );
	alias crc16_ptr : std_logic_vector( 31 downto 0 ) is input_reg( 31 downto 0 );
begin

	crc16_0 : component chc_Configurable_WB_ASP_1_crc16
	port map (
		retval => crc16_retval,
		ptr => crc16_ptr,
		count => crc16_count,
		start => crc16_start,
		done => crc16_done,
		clock => io_CLK_I,
		reset => io_RST_I,
		me_EN => me_0_EN,
		me_DONE => me_0_DONE,
		me_DOUT => me_0_DOUT,
		me_DIN => me_0_DIN,
		me_WE => me_0_WE,
		me_ADR => me_0_ADR,
		me_SEL => me_0_SEL
	);

	DP : process( io_CLK_I, io_RST_I )
	begin
		if io_RST_I = '1' then
			busy_reg <= X"00000000";
			input_reg <= X"0000000000000000";
			me_wbreg_adr <= "000000000000000000000000000000";
			me_wbreg_adr_lsb_reg <= "00";
			me_wbreg_cyc <= '0';
			me_wbreg_in <= X"00000000";
			me_wbreg_sel <= "0000";
			me_wbreg_we <= '0';
			output_reg <= X"0000000000000000";
		elsif rising_edge( io_CLK_I ) then
			if ((io_CYC_I and io_STB_I) and io_WE_I) = '1' then
				if devectorize_bit(io_SEL_I(0 downto 0)) = std_logic'('1') then
					if io_ADR_I = std_logic_vector'("0001") then
						input_reg(7 downto 0) <= io_DAT_I(7 downto 0);
					elsif io_ADR_I = std_logic_vector'("0010") then
						input_reg(39 downto 32) <= io_DAT_I(7 downto 0);
					end if;
				end if;
				if devectorize_bit(io_SEL_I(1 downto 1)) = std_logic'('1') then
					if io_ADR_I = std_logic_vector'("0001") then
						input_reg(15 downto 8) <= io_DAT_I(15 downto 8);
					elsif io_ADR_I = std_logic_vector'("0010") then
						input_reg(47 downto 40) <= io_DAT_I(15 downto 8);
					end if;
				end if;
				if devectorize_bit(io_SEL_I(2 downto 2)) = std_logic'('1') then
					if io_ADR_I = std_logic_vector'("0001") then
						input_reg(23 downto 16) <= io_DAT_I(23 downto 16);
					elsif io_ADR_I = std_logic_vector'("0010") then
						input_reg(55 downto 48) <= io_DAT_I(23 downto 16);
					end if;
				end if;
				if devectorize_bit(io_SEL_I(3 downto 3)) = std_logic'('1') then
					if io_ADR_I = std_logic_vector'("0001") then
						input_reg(31 downto 24) <= io_DAT_I(31 downto 24);
					elsif io_ADR_I = std_logic_vector'("0010") then
						input_reg(63 downto 56) <= io_DAT_I(31 downto 24);
					end if;
				end if;
			end if;
			if crc16_done = '1' then
				busy_reg(0 downto 0) <= vectorize_bit( '0' );
				output_reg(31 downto 16) <= crc16_retval;
			end if;
			if crc16_start = '1' then
				busy_reg(0 downto 0) <= vectorize_bit( '1' );
			end if;
			if me_0_EN = '1' then
				me_wbreg_adr <= me_0_ADR;
				me_wbreg_sel <= me_0_SEL;
				me_wbreg_in <= me_0_DIN;
				me_wbreg_we <= me_0_WE;
				me_wbreg_cyc <= me_0_EN;
				me_wbreg_adr_lsb_reg <= me_wbreg_adr_lsb_wire;
			else
				if me_ACK_I = '1' then
					me_wbreg_adr <= "000000000000000000000000000000";
					me_wbreg_sel <= "0000";
					me_wbreg_in <= X"00000000";
					me_wbreg_we <= '0';
					me_wbreg_cyc <= '0';
					me_wbreg_adr_lsb_reg <= "00";
				end if;
			end if;
		end if; -- reset
	end process; -- DP

	asp_start <= (not busy_reg) and input_start;

	crc16_start <= devectorize_bit( asp_start(0 downto 0) );

	input_start <= io_DAT_I when ( (bool2std_logic((io_ADR_I = std_logic_vector'("0000"))) and ((io_CYC_I and io_STB_I) and io_WE_I)) = '1' )
		else X"00000000";

	io_ACK_O <= io_CYC_I and io_STB_I;

	io_DAT_O_mux : process( busy_reg, io_ADR_I, output_reg )
		type io_DAT_O_mux_select_type is (io_DAT_O_mux_select_type_0, io_DAT_O_mux_select_type_1, io_DAT_O_mux_select_type_2);
	variable io_DAT_O_mux_select : io_DAT_O_mux_select_type := io_DAT_O_mux_select_type_0;
	begin
		if io_ADR_I = std_logic_vector'("0000") then
			io_DAT_O_mux_select := io_DAT_O_mux_select_type_1;
		elsif io_ADR_I = std_logic_vector'("0001") then
			io_DAT_O_mux_select := io_DAT_O_mux_select_type_2;
		else
			io_DAT_O_mux_select := io_DAT_O_mux_select_type_0;
		end if;

		case io_DAT_O_mux_select is
			when io_DAT_O_mux_select_type_1 =>
				io_DAT_O <= not busy_reg;
			when io_DAT_O_mux_select_type_2 =>
				io_DAT_O <= output_reg(31 downto 0);
			when others =>
				io_DAT_O <= output_reg(63 downto 32);
		end case;
	end process; -- io_DAT_O_mux

	me_0_DONE <= me_ACK_I;

	me_0_DOUT <= me_DAT_I;

	me_ADR_O <= me_wbreg_adr & me_wbreg_adr_lsb_reg;

	me_CLK_O <= io_CLK_I;

	me_CYC_O <= me_wbreg_cyc;

	me_DAT_O <= me_wbreg_in;

	me_RST_O <= io_RST_I;

	me_SEL_O <= me_wbreg_sel;

	me_STB_O <= me_wbreg_cyc;

	me_WE_O <= me_wbreg_we;

	me_wbreg_adr_lsb_wire_mux : process( me_0_SEL )
		type me_wbreg_adr_lsb_wire_mux_select_type is (me_wbreg_adr_lsb_wire_mux_select_type_0, me_wbreg_adr_lsb_wire_mux_select_type_1, me_wbreg_adr_lsb_wire_mux_select_type_2, me_wbreg_adr_lsb_wire_mux_select_type_3);
	variable me_wbreg_adr_lsb_wire_mux_select : me_wbreg_adr_lsb_wire_mux_select_type := me_wbreg_adr_lsb_wire_mux_select_type_0;
	begin
		if (me_0_SEL(3 downto 3)) = "1" then
			me_wbreg_adr_lsb_wire_mux_select := me_wbreg_adr_lsb_wire_mux_select_type_1;
		elsif (me_0_SEL(2 downto 2)) = "1" then
			me_wbreg_adr_lsb_wire_mux_select := me_wbreg_adr_lsb_wire_mux_select_type_2;
		elsif (me_0_SEL(1 downto 1)) = "1" then
			me_wbreg_adr_lsb_wire_mux_select := me_wbreg_adr_lsb_wire_mux_select_type_3;
		else
			me_wbreg_adr_lsb_wire_mux_select := me_wbreg_adr_lsb_wire_mux_select_type_0;
		end if;

		case me_wbreg_adr_lsb_wire_mux_select is
			when me_wbreg_adr_lsb_wire_mux_select_type_1 =>
				me_wbreg_adr_lsb_wire <= "00";
			when me_wbreg_adr_lsb_wire_mux_select_type_2 =>
				me_wbreg_adr_lsb_wire <= "01";
			when me_wbreg_adr_lsb_wire_mux_select_type_3 =>
				me_wbreg_adr_lsb_wire <= "10";
			when others =>
				me_wbreg_adr_lsb_wire <= "11";
		end case;
	end process; -- me_wbreg_adr_lsb_wire_mux
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_addi is
generic(
	in1 : in std_logic_vector( 31 downto 0 )
);
port(
	in_x1 : in std_logic_vector( 31 downto 0 );
	retval : out std_logic_vector( 31 downto 0 )
);
end entity chc_Configurable_WB_ASP_1_addi;

architecture rtl of chc_Configurable_WB_ASP_1_addi is
begin

	retval <= std_logic_vector(SIGNED(in_x1) + SIGNED(in1));
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_crc16 is
port(
	clock : in std_logic;
	count : in std_logic_vector( 31 downto 0 );
	done : out std_logic;
	me_ADR : out std_logic_vector( 29 downto 0 );
	me_DIN : out std_logic_vector( 31 downto 0 );
	me_DONE : in std_logic;
	me_DOUT : in std_logic_vector( 31 downto 0 );
	me_EN : out std_logic;
	me_SEL : out std_logic_vector( 3 downto 0 );
	me_WE : out std_logic;
	ptr : in std_logic_vector( 31 downto 0 );
	reset : in std_logic;
	retval : out std_logic_vector( 15 downto 0 );
	start : in std_logic
);
end entity chc_Configurable_WB_ASP_1_crc16;

architecture rtl of chc_Configurable_WB_ASP_1_crc16 is
		type STATE is (STATE_0, STATE_1, STATE_2, STATE_3, STATE_4, STATE_5);
	component chc_Configurable_WB_ASP_1_addi is
	generic(
		in1 : in std_logic_vector( 31 downto 0 )
	);
	port(
		in_x1 : in std_logic_vector( 31 downto 0 );
		retval : out std_logic_vector( 31 downto 0 )
	);
	end component chc_Configurable_WB_ASP_1_addi;

	component chc_Configurable_WB_ASP_1_mux is
	port(
		in1 : in std_logic_vector( 15 downto 0 );
		in_x1 : in std_logic_vector( 15 downto 0 );
		retval : out std_logic_vector( 15 downto 0 );
		sel0 : in std_logic
	);
	end component chc_Configurable_WB_ASP_1_mux;

	component chc_Configurable_WB_ASP_1_nei is
	generic(
		in1 : in std_logic_vector( 3 downto 0 )
	);
	port(
		in_x1 : in std_logic_vector( 3 downto 0 );
		retval : out std_logic
	);
	end component chc_Configurable_WB_ASP_1_nei;

	component chc_Configurable_WB_ASP_1_nei_v2 is
	generic(
		in1 : in std_logic_vector( 31 downto 0 )
	);
	port(
		in_x1 : in std_logic_vector( 31 downto 0 );
		retval : out std_logic
	);
	end component chc_Configurable_WB_ASP_1_nei_v2;

	component chc_Configurable_WB_ASP_1_subui is
	generic(
		in1 : in std_logic_vector( 3 downto 0 )
	);
	port(
		in_x1 : in std_logic_vector( 3 downto 0 );
		retval : out std_logic_vector( 3 downto 0 )
	);
	end component chc_Configurable_WB_ASP_1_subui;

	component chc_Configurable_WB_ASP_1_subui_v2 is
	generic(
		in1 : in std_logic_vector( 31 downto 0 )
	);
	port(
		in_x1 : in std_logic_vector( 31 downto 0 );
		retval : out std_logic_vector( 31 downto 0 )
	);
	end component chc_Configurable_WB_ASP_1_subui_v2;

	component chc_Configurable_WB_ASP_1_vld_me_8 is
	port(
		address : in std_logic_vector( 31 downto 0 );
		clock : in std_logic;
		done : out std_logic;
		me_ADR : out std_logic_vector( 29 downto 0 );
		me_DIN : out std_logic_vector( 31 downto 0 );
		me_DONE : in std_logic;
		me_DOUT : in std_logic_vector( 31 downto 0 );
		me_EN : out std_logic;
		me_SEL : out std_logic_vector( 3 downto 0 );
		me_WE : out std_logic;
		reset : in std_logic;
		retval : out std_logic_vector( 7 downto 0 );
		start : in std_logic
	);
	end component chc_Configurable_WB_ASP_1_vld_me_8;

	component chc_Configurable_WB_ASP_1_xor is
	port(
		in1 : in std_logic_vector( 15 downto 0 );
		in_x1 : in std_logic_vector( 15 downto 0 );
		retval : out std_logic_vector( 15 downto 0 )
	);
	end component chc_Configurable_WB_ASP_1_xor;

	component chc_Configurable_WB_ASP_1_xori is
	generic(
		in1 : in std_logic_vector( 15 downto 0 )
	);
	port(
		in_x1 : in std_logic_vector( 15 downto 0 );
		retval : out std_logic_vector( 15 downto 0 )
	);
	end component chc_Configurable_WB_ASP_1_xori;

	signal addi_0_in : std_logic_vector( 31 downto 0 );
	signal addi_0_retval : std_logic_vector( 31 downto 0 );
	signal current_state : STATE := STATE_0;
	signal mux_0_in : std_logic_vector( 15 downto 0 );
	signal mux_0_in1 : std_logic_vector( 15 downto 0 );
	signal mux_0_retval : std_logic_vector( 15 downto 0 );
	signal mux_0_sel0 : std_logic;
	signal nei_0_in : std_logic_vector( 3 downto 0 );
	signal nei_0_retval : std_logic;
	signal nei_v2_0_in : std_logic_vector( 31 downto 0 );
	signal nei_v2_0_retval : std_logic;
	signal next_state : STATE := STATE_0;
	signal r1 : std_logic_vector( 31 downto 0 ) := X"00000000";
	signal r2 : std_logic_vector( 31 downto 0 ) := X"00000000";
	signal r3 : std_logic_vector( 15 downto 0 ) := X"0000";
	signal r4 : std_logic_vector( 3 downto 0 ) := "0000";
	signal subui_0_in : std_logic_vector( 3 downto 0 );
	signal subui_0_retval : std_logic_vector( 3 downto 0 );
	signal subui_v2_0_in : std_logic_vector( 31 downto 0 );
	signal subui_v2_0_retval : std_logic_vector( 31 downto 0 );
	signal vld_me_8_0_address : std_logic_vector( 31 downto 0 );
	signal vld_me_8_0_done : std_logic;
	signal vld_me_8_0_me_0_ADR : std_logic_vector( 29 downto 0 );
	signal vld_me_8_0_me_0_DIN : std_logic_vector( 31 downto 0 );
	signal vld_me_8_0_me_0_EN : std_logic;
	signal vld_me_8_0_me_0_SEL : std_logic_vector( 3 downto 0 );
	signal vld_me_8_0_me_0_WE : std_logic;
	signal vld_me_8_0_retval : std_logic_vector( 7 downto 0 );
	signal vld_me_8_0_start : std_logic;
	signal xor_0_in : std_logic_vector( 15 downto 0 );
	signal xor_0_in1 : std_logic_vector( 15 downto 0 );
	signal xor_0_retval : std_logic_vector( 15 downto 0 );
	signal xori_0_in : std_logic_vector( 15 downto 0 );
	signal xori_0_retval : std_logic_vector( 15 downto 0 );
begin

	addi_0 : component chc_Configurable_WB_ASP_1_addi
	generic map (
		in1 => X"00000001"
	) 
	port map (
		in_x1 => addi_0_in,
		retval => addi_0_retval
	);

	mux_0 : component chc_Configurable_WB_ASP_1_mux
	port map (
		retval => mux_0_retval,
		in_x1 => mux_0_in,
		sel0 => mux_0_sel0,
		in1 => mux_0_in1
	);

	nei_0 : component chc_Configurable_WB_ASP_1_nei
	generic map (
		in1 => "0000"
	) 
	port map (
		in_x1 => nei_0_in,
		retval => nei_0_retval
	);

	nei_v2_0 : component chc_Configurable_WB_ASP_1_nei_v2
	generic map (
		in1 => X"00000000"
	) 
	port map (
		in_x1 => nei_v2_0_in,
		retval => nei_v2_0_retval
	);

	subui_0 : component chc_Configurable_WB_ASP_1_subui
	generic map (
		in1 => "0001"
	) 
	port map (
		in_x1 => subui_0_in,
		retval => subui_0_retval
	);

	subui_v2_0 : component chc_Configurable_WB_ASP_1_subui_v2
	generic map (
		in1 => X"00000001"
	) 
	port map (
		in_x1 => subui_v2_0_in,
		retval => subui_v2_0_retval
	);

	vld_me_8_0 : component chc_Configurable_WB_ASP_1_vld_me_8
	port map (
		address => vld_me_8_0_address,
		retval => vld_me_8_0_retval,
		start => vld_me_8_0_start,
		done => vld_me_8_0_done,
		clock => clock,
		reset => reset,
		me_EN => vld_me_8_0_me_0_EN,
		me_DONE => me_DONE,
		me_DOUT => me_DOUT,
		me_DIN => vld_me_8_0_me_0_DIN,
		me_WE => vld_me_8_0_me_0_WE,
		me_ADR => vld_me_8_0_me_0_ADR,
		me_SEL => vld_me_8_0_me_0_SEL
	);

	xor_0 : component chc_Configurable_WB_ASP_1_xor
	port map (
		in_x1 => xor_0_in,
		in1 => xor_0_in1,
		retval => xor_0_retval
	);

	xori_0 : component chc_Configurable_WB_ASP_1_xori
	generic map (
		in1 => X"1021"
	) 
	port map (
		in_x1 => xori_0_in,
		retval => xori_0_retval
	);

	SYNC : process( clock, reset )
	begin
		if reset = '1' then
			current_state <= STATE_0;
		elsif rising_edge( clock ) then
			current_state <= next_state;
		end if;
	end process; -- SYNC

	FSM : process( current_state, nei_0_retval, nei_v2_0_retval, start, vld_me_8_0_done )
	begin
		next_state <= current_state;
		case current_state is
		when STATE_0 =>
			if start = '1' then
				next_state <= STATE_1;
			end if;
		when STATE_1 =>
			next_state <= STATE_2;
		when STATE_2 =>
			if vld_me_8_0_done = '1' then
				next_state <= STATE_3;
			end if;
		when STATE_3 =>
			if nei_0_retval = '1' then
				next_state <= STATE_3;
			else
				next_state <= STATE_4;
			end if;
		when STATE_4 =>
			if nei_v2_0_retval = '1' then
				next_state <= STATE_1;
			else
				next_state <= STATE_5;
			end if;
		when STATE_5 =>
			next_state <= STATE_0;
		when others =>
			null;
		end case; -- current_state
	end process; -- FSM

	DP : process( clock, reset )
	begin
		if reset = '1' then
			r1 <= X"00000000";
			r2 <= X"00000000";
			r3 <= X"0000";
			r4 <= "0000";
		elsif rising_edge( clock ) then
			if current_state = STATE_0 then
				r1 <= ptr;
				r2 <= count;
				r3 <= X"0000";
			elsif current_state = STATE_2 then
				if vld_me_8_0_done = '1' then
					r1 <= addi_0_retval;
					r3 <= xor_0_retval;
				end if;
				r4 <= "1000";
			elsif current_state = STATE_3 then
				r3 <= mux_0_retval;
				r4 <= subui_0_retval;
			elsif current_state = STATE_4 then
				r2 <= subui_v2_0_retval;
			end if; -- current_state
		end if; -- reset
	end process; -- DP

	me_EN_resolve : process( vld_me_8_0_me_0_EN )
	begin
		me_EN <= vld_me_8_0_me_0_EN;
	end process; -- me_EN_resolve

	me_DIN_resolve : process( vld_me_8_0_me_0_DIN )
	begin
		me_DIN <= vld_me_8_0_me_0_DIN;
	end process; -- me_DIN_resolve

	me_WE_resolve : process( vld_me_8_0_me_0_WE )
	begin
		me_WE <= vld_me_8_0_me_0_WE;
	end process; -- me_WE_resolve

	me_ADR_resolve : process( vld_me_8_0_me_0_ADR )
	begin
		me_ADR <= vld_me_8_0_me_0_ADR;
	end process; -- me_ADR_resolve

	me_SEL_resolve : process( vld_me_8_0_me_0_SEL )
	begin
		me_SEL <= vld_me_8_0_me_0_SEL;
	end process; -- me_SEL_resolve

	addi_0_in <= r1;

	done <= '1' when ( current_state = STATE_5 )
		else '0';

	mux_0_in <= (r3(14 downto 0)) & std_logic_vector'(vectorize_bit('0'));

	mux_0_in1 <= xori_0_retval;

	mux_0_sel0 <= not devectorize_bit(r3(15 downto 15));

	nei_0_in <= subui_0_retval;

	nei_v2_0_in <= r2;

	retval <= r3;

	subui_0_in <= r4;

	subui_v2_0_in <= r2;

	vld_me_8_0_address <= r1;

	vld_me_8_0_start <= '1' when ( current_state = STATE_1 )
		else '0';

	xor_0_in <= r3;

	xor_0_in1 <= vld_me_8_0_retval & std_logic_vector'(X"00");

	xori_0_in <= (r3(14 downto 0)) & std_logic_vector'(vectorize_bit('0'));
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_mux is
port(
	in1 : in std_logic_vector( 15 downto 0 );
	in_x1 : in std_logic_vector( 15 downto 0 );
	retval : out std_logic_vector( 15 downto 0 );
	sel0 : in std_logic
);
end entity chc_Configurable_WB_ASP_1_mux;

architecture rtl of chc_Configurable_WB_ASP_1_mux is
begin

	retval <= in_x1 when ( sel0 = '1' )
		else in1;
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_nei is
generic(
	in1 : in std_logic_vector( 3 downto 0 )
);
port(
	in_x1 : in std_logic_vector( 3 downto 0 );
	retval : out std_logic
);
end entity chc_Configurable_WB_ASP_1_nei;

architecture rtl of chc_Configurable_WB_ASP_1_nei is
begin

	retval <= bool2std_logic( in_x1 /= in1 );
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_nei_v2 is
generic(
	in1 : in std_logic_vector( 31 downto 0 )
);
port(
	in_x1 : in std_logic_vector( 31 downto 0 );
	retval : out std_logic
);
end entity chc_Configurable_WB_ASP_1_nei_v2;

architecture rtl of chc_Configurable_WB_ASP_1_nei_v2 is
begin

	retval <= bool2std_logic( in_x1 /= in1 );
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_subui is
generic(
	in1 : in std_logic_vector( 3 downto 0 )
);
port(
	in_x1 : in std_logic_vector( 3 downto 0 );
	retval : out std_logic_vector( 3 downto 0 )
);
end entity chc_Configurable_WB_ASP_1_subui;

architecture rtl of chc_Configurable_WB_ASP_1_subui is
begin

	retval <= std_logic_vector(UNSIGNED(in_x1) - UNSIGNED(in1));
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_subui_v2 is
generic(
	in1 : in std_logic_vector( 31 downto 0 )
);
port(
	in_x1 : in std_logic_vector( 31 downto 0 );
	retval : out std_logic_vector( 31 downto 0 )
);
end entity chc_Configurable_WB_ASP_1_subui_v2;

architecture rtl of chc_Configurable_WB_ASP_1_subui_v2 is
begin

	retval <= std_logic_vector(UNSIGNED(in_x1) - UNSIGNED(in1));
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_vld_me_8 is
port(
	address : in std_logic_vector( 31 downto 0 );
	clock : in std_logic;
	done : out std_logic;
	me_ADR : out std_logic_vector( 29 downto 0 );
	me_DIN : out std_logic_vector( 31 downto 0 );
	me_DONE : in std_logic;
	me_DOUT : in std_logic_vector( 31 downto 0 );
	me_EN : out std_logic;
	me_SEL : out std_logic_vector( 3 downto 0 );
	me_WE : out std_logic;
	reset : in std_logic;
	retval : out std_logic_vector( 7 downto 0 );
	start : in std_logic
);
end entity chc_Configurable_WB_ASP_1_vld_me_8;

architecture rtl of chc_Configurable_WB_ASP_1_vld_me_8 is
	signal latched_mau_low : std_logic_vector( 1 downto 0 ) := "00";
begin

	DP : process( clock, reset )
	begin
		if reset = '1' then
			latched_mau_low <= "00";
		elsif rising_edge( clock ) then
			if start = '1' then
				latched_mau_low <= address(1 downto 0);
			end if;
		end if; -- reset
	end process; -- DP

	done <= me_DONE;

	me_ADR <= address(31 downto 2) when ( start = '1' )
		else "000000000000000000000000000000";

	me_DIN <= X"00000000";

	me_EN <= start;

	me_SEL <= "1111" when ( start = '1' )
		else "0000";

	me_WE <= '0';

	retval_mux : process( latched_mau_low, me_DOUT )
		type retval_mux_select_type is (retval_mux_select_type_0, retval_mux_select_type_1, retval_mux_select_type_2, retval_mux_select_type_3);
	variable retval_mux_select : retval_mux_select_type := retval_mux_select_type_0;
	begin
		if latched_mau_low = std_logic_vector'("00") then
			retval_mux_select := retval_mux_select_type_1;
		elsif latched_mau_low = std_logic_vector'("01") then
			retval_mux_select := retval_mux_select_type_2;
		elsif latched_mau_low = std_logic_vector'("10") then
			retval_mux_select := retval_mux_select_type_3;
		else
			retval_mux_select := retval_mux_select_type_0;
		end if;

		case retval_mux_select is
			when retval_mux_select_type_1 =>
				retval <= me_DOUT(31 downto 24);
			when retval_mux_select_type_2 =>
				retval <= me_DOUT(23 downto 16);
			when retval_mux_select_type_3 =>
				retval <= me_DOUT(15 downto 8);
			when others =>
				retval <= me_DOUT(7 downto 0);
		end case;
	end process; -- retval_mux
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_xor is
port(
	in1 : in std_logic_vector( 15 downto 0 );
	in_x1 : in std_logic_vector( 15 downto 0 );
	retval : out std_logic_vector( 15 downto 0 )
);
end entity chc_Configurable_WB_ASP_1_xor;

architecture rtl of chc_Configurable_WB_ASP_1_xor is
begin

	retval <= in_x1 xor in1;
end rtl;


	library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	use work.Configurable_WB_ASP_1_chc_pkg.all;

entity chc_Configurable_WB_ASP_1_xori is
generic(
	in1 : in std_logic_vector( 15 downto 0 )
);
port(
	in_x1 : in std_logic_vector( 15 downto 0 );
	retval : out std_logic_vector( 15 downto 0 )
);
end entity chc_Configurable_WB_ASP_1_xori;

architecture rtl of chc_Configurable_WB_ASP_1_xori is
begin

	retval <= in_x1 xor in1;
end rtl;

	-- end vhdl
