--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
Entity Configurable_WB_INTERCON_1 Is
  Port(
        s0_STB_O   : Out   Std_Logic;
        s0_CYC_O   : Out   Std_Logic;
        s0_ACK_I   : In    Std_Logic;
        s0_DAT_I   : In    Std_Logic_Vector(7 DownTo 0);
        s0_DAT_O   : Out   Std_Logic_Vector(7 DownTo 0);
        s0_SEL_O   : Out   Std_Logic_Vector(3 DownTo 0);
        s0_WE_O    : Out   Std_Logic;

        s1_STB_O   : Out   Std_Logic;
        s1_CYC_O   : Out   Std_Logic;
        s1_ACK_I   : In    Std_Logic;
        s1_ADR_O   : Out   Std_Logic_Vector(3 DownTo 0);
        s1_DAT_I   : In    Std_Logic_Vector(7 DownTo 0);
        s1_DAT_O   : Out   Std_Logic_Vector(7 DownTo 0);
        s1_SEL_O   : Out   Std_Logic_Vector(3 DownTo 0);
        s1_WE_O    : Out   Std_Logic;

        s2_STB_O   : Out   Std_Logic;
        s2_CYC_O   : Out   Std_Logic;
        s2_ACK_I   : In    Std_Logic;
        s2_ADR_O   : Out   Std_Logic_Vector(3 DownTo 0);
        s2_DAT_I   : In    Std_Logic_Vector(31 DownTo 0);
        s2_DAT_O   : Out   Std_Logic_Vector(31 DownTo 0);
        s2_SEL_O   : Out   Std_Logic_Vector(3 DownTo 0);
        s2_WE_O    : Out   Std_Logic;

        m0_STB_I   : In    Std_Logic;
        m0_CYC_I   : In    Std_Logic;
        m0_ACK_O   : Out   Std_Logic;
        m0_ADR_I   : In    Std_Logic_Vector(23 DownTo 0);
        m0_DAT_O   : Out   Std_Logic_Vector(31 DownTo 0);
        m0_DAT_I   : In    Std_Logic_Vector(31 DownTo 0);
        m0_SEL_I   : In    Std_Logic_Vector(3 DownTo 0);
        m0_WE_I    : In    Std_Logic;
        m0_CLK_I   : In    Std_Logic
      );
End Configurable_WB_INTERCON_1;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
Architecture RTL Of Configurable_WB_INTERCON_1 Is
    Constant s0_DecodeAddress     : Std_Logic_Vector(7 DownTo 0) := "00000000";
    Constant s1_DecodeAddress     : Std_Logic_Vector(7 DownTo 0) := "00000001";
    Constant s2_DecodeAddress     : Std_Logic_Vector(7 DownTo 0) := "00000010";

    Signal   s0_Decode            : Std_Logic;
    Signal   s1_Decode            : Std_Logic;
    Signal   s2_Decode            : Std_Logic;

    Signal   s0_DAT               : Std_Logic_Vector(31 DownTo 0);
    Signal   s1_DAT               : Std_Logic_Vector(31 DownTo 0);
    Signal   s2_DAT               : Std_Logic_Vector(31 DownTo 0);

    Signal   Any_Decode           : Std_Logic;
    Signal   Bad_Decode           : Std_Logic;
    Signal   UseBadDecode         : Std_Logic;

    Signal   DecodeAddr           : Std_Logic_Vector(7 DownTo 0);

Begin
    s0_Decode <= '1' When m0_ADR_I(23 DownTo 16) = s0_DecodeAddress Else '0';
    s1_Decode <= '1' When m0_ADR_I(23 DownTo 16) = s1_DecodeAddress Else '0';
    s2_Decode <= '1' When m0_ADR_I(23 DownTo 16) = s2_DecodeAddress Else '0';

    s0_STB_O <= m0_STB_I And s0_Decode And Not Bad_Decode;
    s0_CYC_O <= m0_CYC_I And s0_Decode And Not Bad_Decode;
    s0_DAT_O <= m0_DAT_I(7 Downto 0);
    s0_SEL_O <= m0_SEL_I;
    s0_WE_O  <= m0_WE_I;

    s1_ADR_O <= m0_ADR_I(3 Downto 0);
    s1_STB_O <= m0_STB_I And s1_Decode And Not Bad_Decode;
    s1_CYC_O <= m0_CYC_I And s1_Decode And Not Bad_Decode;
    s1_DAT_O <= m0_DAT_I(7 Downto 0);
    s1_SEL_O <= m0_SEL_I;
    s1_WE_O  <= m0_WE_I;

    s2_ADR_O <= m0_ADR_I(5 Downto 2);
    s2_STB_O <= m0_STB_I And s2_Decode And Not Bad_Decode;
    s2_CYC_O <= m0_CYC_I And s2_Decode And Not Bad_Decode;
    s2_DAT_O <= m0_DAT_I(31 Downto 0);
    s2_SEL_O <= m0_SEL_I;
    s2_WE_O  <= m0_WE_I;

    Any_Decode <= 
          s0_Decode Or
          s1_Decode Or
          s2_Decode;

    process(m0_CLK_I)
    begin
        if rising_edge(m0_CLK_I) then
            if Any_Decode='0' then
              DecodeAddr <= m0_ADR_I(23 Downto 16);
            end if;
            if DecodeAddr /= m0_ADR_I(23 Downto 16) then
              UseBadDecode <= '1';
            else
              UseBadDecode <= '0';
            end if;
            Bad_Decode <= Not(Any_Decode) And m0_STB_I And m0_CYC_I;
        end if;
    end process;


    s0_DAT     <= s0_DAT_I & s0_DAT_I & s0_DAT_I & s0_DAT_I When s0_Decode = '1' Else (Others => '0');

    s1_DAT     <= s1_DAT_I & s1_DAT_I & s1_DAT_I & s1_DAT_I When s1_Decode = '1' Else (Others => '0');

    s2_DAT     <= s2_DAT_I When s2_Decode = '1' Else (Others => '0');

    m0_ACK_O <= (Bad_Decode And UseBadDecode) Or
                s0_ACK_I   Or
                s1_ACK_I   Or
                s2_ACK_I  ;

    m0_DAT_O <= 
                s0_DAT     Or
                s1_DAT     Or
                s2_DAT    ;

End RTL;
--------------------------------------------------------------------------------
